public class Simple_notifying_queue 
{
    private static final int QUEUE_SIZE = 10;
    private Object[]         queue      = new Object[ QUEUE_SIZE ];
    private int              head       = 0;
    private int              tail       = 0;

    public synchronized void enqueue( Object item )
    {
        tail          = ++tail % QUEUE_SIZE;
        queue[ tail ] = item; 
        this.notify();                  // The "this" is there only to
    }                                   // improve readability.

    public synchronized Object dequeue( )
    {
        try
        {   // If the queue is empty, wait for some other thread to
            // enqueue something. The following test MUST be a while,
            // not an if, statement.
            
            while( head == tail )
                this.wait();        
        }
        catch( InterruptedException e )
        {
            // If we get here, we were not actually notified.
            // returning null doesn't indicate that the
            // queue is empty, only that the waiting was
            // abandoned.
    
            return null;
        }

        // Remove the object from the queue. Note that the assignment
        // of null to the vacated slot is essential. If I didn't do
        // it, the reference to the dequeued object could stay alive
        // until the queue is garbage collected, even though the object
        // has been "removed" from the queue. Who says there's no such
        // thing as a memory leak in Java?

        head                    = ++head % QUEUE_SIZE;
        Object dequeued_object  = queue[ head ]; 
        queue[head]             = null;
        return dequeued_object;
    }
}
